!Fortran90 program (using the NAG library for chi-square probabilities)

!This simple program assesses how consistent a log-normal 'observed' 
!dataset is with populations of log-normal or exponential data,
!producing the results in Table 2, line (a)

!Elsewhere, data are simulated from the models and the
!likelihood evaluated or maximised for each simulated dataset

!This program reads in likelihoods from file 'lhoods.dat'
!MODEL 1 is log-normal, MODEL 2 is exponential model
!Idata corresponds to maximized likelihoods
!Sdata corresponds to likelihoods
!For both Idata and Sdata
!-- the first column (:,1) is likelihood under the log-normal model
!   and the second (:,2) is under the exponential model
!-- lines 1-100 (1:100,:) are when the simulated data are log-normal
!   lines 101-200 (101:200,:) when exponential
!   line 201 (102,:) carries the info for the 'observed' data itself,
!   here the observed data, i.e. correct model is log-normal


MODULE dp
  IMPLICIT NONE
  INTEGER, PARAMETER :: idp = KIND(1.0D0) ! defines double precision
END MODULE dp


PROGRAM mah
  USE dp
  IMPLICIT NONE
  INTEGER, DIMENSION(201) :: MODEL
  REAL(idp), DIMENSION(201,2) :: Idata,Sdata,dataC 

  REAL(idp), DIMENSION(100,100),PARAMETER :: one=1.
  REAL(idp), DIMENSION(2,100) :: dataCtr
  REAL(idp), DIMENSION(1,2) :: mean,x0
  REAL(idp), DIMENSION(2,1) :: x0T
  REAL(idp), DIMENSION(2,2) :: cov,inv
  REAL(idp), DIMENSION(1,1) :: D1
  REAL(idp) :: det,p
  INTEGER :: i,info
  REAL(idp), PARAMETER :: df=2
  REAL(idp), EXTERNAL :: G01ECF !chi-square probability

  !read in one of 3 output files depending on what correct model is 
  !this version for (a) log-normal - see above
  OPEN(5,FILE='lhoods.dat') 
  READ(5,*) (MODEL(i),Idata(i,1),Idata(i,2),Sdata(i,1),Sdata(i,2),i=1,201)
  CLOSE(5)


  !A - for model 1(LN), data I (maximized likelihoods)
  WRITE(*,*) 'Test against LN model using maximised lh '

  !calculate means
  mean(1,1) = sum(Idata(1:100,1))/100.
  mean(1,2) = sum(Idata(1:100,2))/100.
  WRITE(*,*) 'means '
  WRITE(*,'(2(F10.3))') mean(1,:)
  
  !centre data and calculate covariance
  dataC(1:100,:) = Idata(1:100,:)-(matmul(one,Idata(1:100,:))/100.)
  dataCtr(:,1:100) = transpose(dataC(1:100,:))
  cov(:,:) = matmul(dataCtr,dataC(1:100,:))/100.
  WRITE(*,*) 'cov'
  WRITE(*,'(2(F10.3))') cov

  !invert covariance
  det = cov(1,1)*cov(2,2) - cov(1,2)*cov(2,1)
  inv(1,1) = cov(2,2)
  inv(2,2) = cov(1,1)
  inv(1,2) = -cov(1,2)
  inv(2,1) = -cov(2,1)
  inv(:,:) = inv(:,:)/det
  WRITE(*,*) 'inverse'
  WRITE(*,'(2(F10.3))') inv
  
  x0(1,:) = Idata(201,:)-mean(1,:)
  x0T = transpose(x0)

  !mahalanobis distance and p-value
  D1 = matmul(matmul(x0,inv),x0T)
  p = G01ECF('U',D1,df,info) !chi-sq
  WRITE(*,'(A16,F8.2,F7.3,/)') 'D1 and p-value ',D1,p


  !B - for model 2(exp), data I (maximized likelihoods)
  WRITE(*,*) 'Test against EXP model using maximised lh '

  !calculate means
  mean(1,1) = sum(Idata(101:200,1))/100.
  mean(1,2) = sum(Idata(101:200,2))/100.
  WRITE(*,*) 'means '
  WRITE(*,'(2(F10.4))') mean(1,:)
  
  !centre data and calculate covariance
  dataC(101:200,:) = Idata(101:200,:)-(matmul(one,Idata(101:200,:))/100.)
  dataCtr(:,1:100) = transpose(dataC(101:200,:))
  cov(:,:) = matmul(dataCtr,dataC(101:200,:))/100.
  WRITE(*,*) 'cov'
  WRITE(*,'(2(F10.4))') cov

  !invert covariance
  det = cov(1,1)*cov(2,2) - cov(1,2)*cov(2,1)
  inv(1,1) = cov(2,2)
  inv(2,2) = cov(1,1)
  inv(1,2) = -cov(1,2)
  inv(2,1) = -cov(2,1)
  inv(:,:) = inv(:,:)/det
  WRITE(*,*) 'inverse'
  WRITE(*,'(2(F10.3))') inv
  
  x0(1,:) = Idata(201,:)-mean(1,:)
  x0T = transpose(x0)

  !mahalanobis distance and p-value
  D1 = matmul(matmul(x0,inv),x0T)
  p = G01ECF('U',D1,df,info)
  WRITE(*,'(A16,F8.2,F7.3,/)') 'D1 and p-value ',D1,p


  !C - for model 1(LN), data S (likelihoods)
  WRITE(*,*) 'Test against LN model using lh '

  !calculate means
  mean(1,1) = sum(Sdata(1:100,1))/100.
  mean(1,2) = sum(Sdata(1:100,2))/100.
  WRITE(*,*) 'means '
  WRITE(*,'(2(F10.4))') mean(1,:)
  
  !centre data and calculate covariance
  dataC(1:100,:) = Sdata(1:100,:)-(matmul(one,Sdata(1:100,:))/100.)
  dataCtr(:,1:100) = transpose(dataC(1:100,:))
  cov(:,:) = matmul(dataCtr,dataC(1:100,:))/100.
  WRITE(*,*) 'cov'
  WRITE(*,'(2(F10.4))') cov

  !invert covariance
  det = cov(1,1)*cov(2,2) - cov(1,2)*cov(2,1)
  inv(1,1) = cov(2,2)
  inv(2,2) = cov(1,1)
  inv(1,2) = -cov(1,2)
  inv(2,1) = -cov(2,1)
  inv(:,:) = inv(:,:)/det
  WRITE(*,*) 'inverse'
  WRITE(*,'(2(F10.3))') inv
  
  x0(1,:) = Sdata(201,:)-mean(1,:)
  x0T = transpose(x0)

  !mahalanobis distance and p-value
  D1 = matmul(matmul(x0,inv),x0T)
  p = G01ECF('U',D1,df,info)
  WRITE(*,'(A16,F8.2,F7.3,/)') 'D1 and p-value ',D1,p


  !D - for model 2(exp), data S (likelihoods)
  WRITE(*,*) 'Test against EXP model using lh '

  !calculate means
  mean(1,1) = sum(Sdata(101:200,1))/100.
  mean(1,2) = sum(Sdata(101:200,2))/100.
  WRITE(*,*) 'means '
  WRITE(*,'(2(F10.4))') mean(1,:)
  
  !centre data and calculate covariance
  dataC(101:200,:) = Sdata(101:200,:)-(matmul(one,Sdata(101:200,:))/100.)
  dataCtr(:,1:100) = transpose(dataC(101:200,:))
  cov(:,:) = matmul(dataCtr,dataC(101:200,:))/100.
  WRITE(*,*) 'cov'
  WRITE(*,'(2(F10.4))') cov

  !invert covariance
  det = cov(1,1)*cov(2,2) - cov(1,2)*cov(2,1)
  inv(1,1) = cov(2,2)
  inv(2,2) = cov(1,1)
  inv(1,2) = -cov(1,2)
  inv(2,1) = -cov(2,1)
  inv(:,:) = inv(:,:)/det
  
  x0(1,:) = Sdata(201,:)-mean(1,:)
  x0T = transpose(x0)

  !mahalanobis distance and p-value
  D1 = matmul(matmul(x0,inv),x0T)
  p = G01ECF('U',D1,df,info)
  WRITE(*,'(A16,F8.2,F7.3)') 'D1 and p-value ',D1,p

END PROGRAM mah
